/*  _null.h

    Definition of NULL.

*/

/*
 *      C/C++ Run Time Library - Version 6.0
 *
 *      Copyright (c) 1987, 1993 by Borland International
 *      All Rights Reserved.
 *
 */

#ifdef __BORLANDC__
#include <_null.h>
#else

#ifndef TVISION_COMPAT__NULL_H
#define TVISION_COMPAT__NULL_H

#include <stddef.h>

#endif // TVISION_COMPAT__NULL_H

#endif // __BORLANDC__
