/*-----------------------------------------------------*/
/*                                                     */
/*   Tvcmds.h : Constants for TVDemo events            */
/*                                                     */
/*-----------------------------------------------------*/
/*
 *      Turbo Vision - Version 2.0
 *
 *      Copyright (c) 1994 by Borland International
 *      All Rights Reserved.
 *
 */

#if !defined( __TVCMDS_H )
#define __TVCMDS_H

const int cmAboutCmd    = 100;
const int cmPuzzleCmd   = 101;
const int cmCalendarCmd = 102;
const int cmAsciiCmd    = 103;
const int cmCalcCmd     = 104;
const int cmOpenCmd     = 105;
const int cmChDirCmd    = 106;
const int cmMouseCmd    = 108;
const int cmColorCmd    = 109;
const int cmSaveCmd     = 110;
const int cmRestoreCmd  = 111;
const int cmEventViewCmd= 112;
const int cmChBackground= 113;
const int cmFndEventView= 114;
const int cmVideoMode   = 2002;

#endif // __TVCMDS_H
