/*------------------------------------------------------------*/
/* filename -       nmdircol.cpp                              */
/*                                                            */
/* defines the streamable name for class TDirCollection       */
/*------------------------------------------------------------*/
/*
 *      Turbo Vision - Version 2.0
 *
 *      Copyright (c) 1994 by Borland International
 *      All Rights Reserved.
 *
 */

#define Uses_TDirCollection
#include <tvision/tv.h>

const char * const _NEAR TDirCollection::name = "TDirCollection";
